local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:AUCTION_HOUSE_SHOW()
	self:DebugEntry("AUCTION_HOUSE_SHOW")
	self:UpdateRestockStack()
	self:UpdateAuctionUI()
	self:UpdateComponentUI()
	self:DebugExit("AUCTION_HOUSE_SHOW")
end

function Capitalist:AUCTION_OWNED_LIST_UPDATE()
	self:DebugEntry("AUCTION_OWNED_LIST_UPDATE")
	self:UpdateAuctionUI()
	self:DebugExit("AUCTION_OWNED_LIST_UPDATE")
end

function Capitalist:BAG_UPDATE(...)
	self:DebugEntry("BAG_UPDATE")

	local event, arg1 = ...;
	self:DebugMisc(arg1)

	-- save all products and components in that players inventory
	local itemsInInv = self.db.profile.storage[realm][character].inventory
	for k, v in pairs(itemsInInv) do itemsInInv[k] = nil end
	
	-- REVIEW: Very similar code in BANKFRAME_CLOSED. Helper function?
	for bag = 0, 4 do
		for slot = 1, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if ((self.list and self.list[name]) or
					(self.db.profile.components and self.db.profile.components[name])) then
					local _, count = GetContainerItemInfo(bag, slot)
					
					if (not itemsInInv[name]) then
						itemsInInv[name] = count
					else
						itemsInInv[name] = itemsInInv[name] + count
					end
				end
			end
		end
	end

	if (self.mode == "posting") then
		if (self.PostingPostActive) then
			Capitalist:PostCallback(true)
		end
		if (self.PostingSplitActive) then
			self:DebugMisc("Q")
			Capitalist:SplitCallback()
		end
	end

	self:DebugExit("BAG_UPDATE")
end

function Capitalist:CHAT_MSG_LOOT(...)
	self:DebugEntry("CHAT_MSG_LOOT")

	local event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11 = ...;
	self:DebugMisc(arg1)	--You create: [Scroll of Enchant Chest - Powerful Stats].
	self:DebugMisc(arg7)	--0
	self:DebugMisc(arg8)	--0
	self:DebugMisc(arg9)	--(empty string)
	self:DebugMisc(arg10)	--0
	self:DebugMisc(arg11)	--1123

	if (string.find(arg1, L["You create:"])) then
		local item = string.match(arg1, "%[(.*)%]")
		self:DebugMisc(item)

		if (item and self.list[item]) then
			item = self.list[item].skillName
			self.restock[item] = self.restock[item] - 1
			if (self.restock[item] <= 0) then
				self.restock[item] = nil
			end

			local skillName = item
			if (L.isScrollOfEnchant(item)) then
				skillName = L.stripScrollPrefix(item)
				self:DebugMisc(skillName)	--Enchant Chest - Powerful Stats

				local profession = L["Enchanting"]
				self.db.profile[realm][character][profession][skillName] = self.db.profile[realm][character][profession][skillName] + 1
			else
				-- TODO: other professions
			end

			-- TODO: enchanting-specific
			self:UpdateStatistics("Enchanting")
		end

		self.inTradeskill = false
	end

	self:DebugExit("CHAT_MSG_LOOT")
end

function Capitalist:CHAT_MSG_SYSTEM(...)
	self:DebugEntry("CHAT_MSG_SYSTEM")

	local event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11 = ...;
	self:DebugMisc(arg1)	--Auction created.

	if (string.find(arg1, L["Auction created."])) then
		if (self.PostingPostActive) then
			Capitalist:PostCallback(true)
		end
	end

	self:DebugExit("CHAT_MSG_SYSTEM")
end

function Capitalist:UNIT_SPELLCAST_INTERRUPTED(...)
	self:DebugEntry("UNIT_SPELLCAST_INTERRUPTED")

	local event, arg1, arg2, arg3, arg4, arg5 = ...;
	self:DebugMisc(arg1)
	self:DebugMisc(arg2)
	self:DebugMisc(arg4)

	if (arg1 == "player") then
		self.inTradeskill = false
	end

	self:DebugExit("UNIT_SPELLCAST_INTERRUPTED")
end

function Capitalist:UI_ERROR_MESSAGE(...)
	self:DebugEntry("UI_ERROR_MESSAGE")

	local event, arg1 = ...;
	self:DebugMisc(arg1)

	if (arg1 == L["Inventory is full."]) then
		self.inTradeskill = false
	end

	self:DebugExit("UI_ERROR_MESSAGE")
end

function Capitalist:BANKFRAME_OPENED(...)
	self:DebugEntry("BANKFRAME_OPENED")
	self.bankFrameOpen = true
	self:DebugExit("BANKFRAME_OPENED")
end

function Capitalist:BANKFRAME_CLOSED(...)
	self:DebugEntry("BANKFRAME_CLOSED")

	if (not self.bankFrameOpen) then return end
	self.bankFrameOpen = false

	local itemsInBank = self.db.profile.storage[realm][character].bank
	for k, v in pairs(itemsInBank) do itemsInBank[k] = nil end
	
	local containersToCheck = {-1, 5, 6, 7, 8, 9, 10, 11}
	
	for _, bag in pairs(containersToCheck) do
		for slot = 1, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (self.list[name] or self.db.profile.components[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					
					if (not itemsInBank[name]) then
						itemsInBank[name] = count
					else
						itemsInBank[name] = itemsInBank[name] + count
					end
				end
			end
		end
	end

	self:DebugExit("BANKFRAME_CLOSED")
end
