local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:UpdateRestockStack()
	self:DebugEntry("UpdateRestockStack")

	self.inTradeskill = false

	self.restock = {}

	if (self.db.profile[realm] and
		self.db.profile[realm][character]) then

		-- TODO: enchanting-specific
		local profession = L["Enchanting"]

		if (self.db.profile[realm][character][profession]) then
			for skillName, numCreated in pairs(self.db.profile[realm][character][profession]) do
				local product = self.db.profile.professions[profession][skillName]

				if (product and
					product.count and
					self:ProductIsEnabled(profession, skillName) and
					self:ProductIsVisible(profession, skillName) and
					product.count > 0) then
					local count = product.count

					-- search AH and all inventories on that server for this product
					for player, t in pairs(self.db.profile.storage[realm]) do
						if (t.AH[product.scroll]) then
							count = count - t.AH[product.scroll]
						end
						if (t.inventory[product.scroll]) then
							count = count - t.inventory[product.scroll]
						end
					end
			
					if (count > 0) then
						self.restock[skillName] = count
					end
				end
			end
	
			self.requiredComponents = {}

			for skillName, count in pairs(self.restock) do
				local product = self.db.profile.professions[profession][skillName]

				for componentName, componentCount in pairs(product.components) do
					if (self.requiredComponents[componentName] == nil) then
						self.requiredComponents[componentName] = componentCount * count
					else
						self.requiredComponents[componentName] = componentCount * count + self.requiredComponents[componentName]
					end
				end
			end
		end
	end

	self:DebugExit("UpdateRestockStack")
end

function Capitalist:ShowRestockStack()
	self:DebugEntry("ShowRestockStack")

	self:UpdateRestockStack()

	local list = {}
	local numRestock = 0
	local components = {}
	for k, v in pairs(self.requiredComponents) do
		components[k] = v
	end
	
	for skillName, count in pairs(self.restock) do
		list[skillName] = count
		numRestock = numRestock + count
	end
	
	-- search for the required components in our inventory
	for bag = 0, 4 do
		for slot = 0, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (components[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					components[name] = components[name] - count
				end
			end
		end
	end
	
	-- print out the restock stack
	if (numRestock == 0) then
		self:Print(L["Nothing to restock!"])
		return
	else
		self:Print(string.format(L["Need to restock %d products:"], numRestock))
		for item, count in pairs(list) do
			ChatFrame1:AddMessage("  "..count.."x "..item)
		end
	end
	
	-- print out any components missing
	list = {}
	local numMissing = 0
	for componentName, componentCount in pairs(components) do
		if (componentCount > 0) then
			list[componentName] = componentCount
			numMissing = numMissing + componentCount
		end
	end
	
	if (numMissing == 0) then
		self:Print(L["Everything available!"])
	else
		self:Print(L["Missing components:"])
		for item, count in pairs(list) do
			ChatFrame1:AddMessage("  "..count.."x "..item)
		end
	end

	self:DebugExit("ShowRestockStack")
end

function Capitalist:Restock(profession)
	self:DebugEntry("Restock")

	if (self.inTradeskill) then
		return
	end

	SetTradeSkillSubClassFilter(0, 1, 1)
	SetTradeSkillInvSlotFilter(0, 1, 1)
	-- REVIEW: This had errors for non-Enchanting professions
	--ExpandTradeSkillSubClass(1)
	SetTradeSkillItemNameFilter("")
	
	for k, v in pairs(self.restock) do
		if (self:TryRestock(profession, k)) then
			return
		end
	end
	
	self:Print(L["Could not restock anything!"])

	self:DebugExit("Restock")
end

function Capitalist:TryRestock(profession, product)
	self:DebugEntry("TryRestock")
	self:DebugMisc(profession)
	self:DebugMisc(product)

	local bag
	local slot

	local isEnchanting = (profession == L["Enchanting"])

	if (isEnchanting) then
		local vellum = L["Enchanting Vellum"];

		for i = 0, 4 do
			for j = 1, GetContainerNumSlots(i) do
				local link = GetContainerItemLink(i, j)
				if (link) then
					local name = link:match("%[(.*)%]")
					if (name == vellum) then
						bag = i
						slot = j
					end
				end
			end
		end

		if (not bag) then
			return false
		end
	end

	for i = 1, GetNumTradeSkills() do
		local skill, _, numAvailable = GetTradeSkillInfo(i)
		if ((skill == product) and (numAvailable > 0)) then
			self:DebugMisc(skill)

			-- REVIEW: Opportunity for multi
			DoTradeSkill(i)
			if (isEnchanting) then
				UseContainerItem(bag, slot)
			end
			self.inTradeskill = true
			return self:DebugExit("TryRestock", true)
		end
	end

	return self:DebugExit("TryRestock", false)
end
